@@ -5,8 +5,10 @@
 from . import number_funcs
 from . import string_funcs
 from . import datetime_funcs
+from . import json_funcs
 
 funcs = {}
 funcs.update(number_funcs.funcs)
 funcs.update(string_funcs.funcs)
-funcs.update(datetime_funcs.funcs)
\ No newline at end of file
+funcs.update(datetime_funcs.funcs)
+funcs.update(json_funcs.funcs)
\ No newline at end of file
