@@ -3,7 +3,6 @@
 # create by: snower
 
 import os
-import sys
 import copy
 import re
 import uuid
@@ -12,6 +11,7 @@ import json
 import pytz
 from syncany.taskers.core import CoreTasker
 from syncany.taskers.config import load_config
+from syncany.logger import get_logger
 from syncany.utils import set_timezone
 
 
@@ -24,6 +24,11 @@ class GlobalConfig(object):
     def __init__(self):
         self.config = copy.deepcopy(CoreTasker.DEFAULT_CONFIG)
 
+    def get_home(self):
+        if "SYNCANY_HOME" in os.environ:
+            return os.path.abspath(os.environ["SYNCANY_HOME"])
+        return os.path.abspath(os.path.join(os.path.expanduser('~'), ".syncany"))
+
     def get(self):
         return self.config
 
@@ -121,9 +126,9 @@ class GlobalConfig(object):
         return query, args
 
     def load(self, custom_config=None):
-        home_config_path = os.path.join(os.path.expanduser('~'), ".syncany")
-        for filename in (os.path.join(home_config_path, "config.json"), os.path.join(home_config_path, "config.yaml"),
-                         "config.json", "config.yaml"):
+        home_config_path, cwd_config_path = self.get_home(), os.path.abspath(os.getcwd())
+        for filename in {os.path.join(home_config_path, "config.json"), os.path.join(home_config_path, "config.yaml"),
+                         os.path.join(cwd_config_path, "config.json"), os.path.join(cwd_config_path, "config.yaml")}:
             if not os.path.exists(filename):
                 continue
             if "extends" not in self.config or not isinstance(self.config["extends"], list):
@@ -131,11 +136,11 @@ class GlobalConfig(object):
             if filename not in self.config["extends"]:
                 self.config["extends"].append(filename)
         self.load_config()
-        if custom_config and isinstance(custom_config):
+        if custom_config and isinstance(custom_config, dict):
             self.merge_config(custom_config)
 
         if "timezone" in self.config:
-            timezone = self.config.pop("timezone")
+            timezone = self.config.get("timezone")
             set_timezone(pytz.timezone(timezone))
         if "databases" not in self.config:
             self.config["databases"] = []
@@ -151,10 +156,10 @@ class GlobalConfig(object):
                 if key not in databases["--"]:
                     databases["--"][key] = value
 
-        encoding = self.config.pop("encoding", None)
-        datetime_format = self.config.pop("datetime_format", None)
-        date_format = self.config.pop("date_format", None)
-        time_format = self.config.pop("time_format", None)
+        encoding = self.config.get("encoding", None)
+        datetime_format = self.config.get("datetime_format", None)
+        date_format = self.config.get("date_format", None)
+        time_format = self.config.get("time_format", None)
         for database in self.config["databases"]:
             if database["driver"] not in ("textline", "json", "csv"):
                 continue
@@ -185,6 +190,16 @@ class GlobalConfig(object):
             self.load_config(extends)
         self.merge_config(config)
 
+    def load_extensions(self):
+        extensions = self.config.get("extensions")
+        if not extensions or not isinstance(extensions, list):
+            return
+        for extension in extensions:
+            try:
+                __import__(extension, globals(), locals(), [extension.rpartition(".")[-1]])
+            except Exception as e:
+                get_logger().warning("import extension %s error %s:%s", extension, e.__class__.__name__, str(e))
+
     def merge_config(self, config):
         for k, v in config.items():
             if k in ("arguments", "imports", "defines", "variables", "sources", "logger", "options"):
@@ -227,11 +242,11 @@ class GlobalConfig(object):
                 self.config[k] = v
 
     def config_logging(self, isatty=True):
-        logfile = self.config.pop("logfile", None)
+        logfile = self.config.get("logfile", None)
         if not logfile or logfile == "-":
             logfile = None
-        logformat = self.config.pop("logformat", "%(asctime)s %(process)d %(levelname)s %(message)s")
-        loglevel = self.config.pop("loglevel", None)
+        logformat = self.config.get("logformat", "%(asctime)s %(process)d %(levelname)s %(message)s")
+        loglevel = self.config.get("loglevel", None)
         if "logger" in self.config and isinstance(self.config["logger"], dict):
             logging.config.dictConfig(self.config["logger"])
         else:
