@@ -1,20 +1,20 @@
 # -*- coding: utf-8 -*-
 # 2023/2/13
 # create by: snower
-
+import datetime
 import os
 import sys
 import re
-import copy
+import threading
 from collections import deque
 from syncany.filters import StringFilter
 from syncany.calculaters import find_calculater
-from syncany.database.memory import MemoryDBFactory, MemoryDBCollection
+from syncany.database import find_database
 from .errors import SyncanySqlCompileException
 from .utils import parse_value
 from .compiler import Compiler
 
-ENV_VARIABLE_RE = re.compile("(\$\{\w+?(:.*?){0,1}\})", re.DOTALL | re.M)
+ENV_VARIABLE_RE = re.compile("(\$\{[@\w]+?(:.*?){0,1}\})", re.DOTALL | re.M)
 RAW_SQL_RE = re.compile("(\/\*\s*raw\(([\w\.]+?)\)\s*(\*\/\s*\()?\s*(.*?)\s*(\)\s*\/\*)?\s*endraw\s*\*\/)", re.DOTALL | re.M)
 FUNC_RE = re.compile("^(\w+?)\(((.+),{0,1})*\)$", re.DOTALL)
 
@@ -26,11 +26,11 @@ class EnvVariables(dict):
         self.parent = parent
 
     @classmethod
-    def build_global(cls):
+    def build_global(cls, session_config):
         env_variables = cls()
         env_variables.update({key.lower(): value for key, value in os.environ.items()})
         env_variables["home"] = os.path.expanduser('~')
-        env_variables["syncany_home"] = os.path.join(os.path.expanduser('~'), ".syncany")
+        env_variables["syncany_home"] = session_config.get_home()
         env_variables["cwd"] = os.getcwd()
         env_variables["platform"] = sys.platform
         for arg in sys.argv:
@@ -63,8 +63,13 @@ class EnvVariables(dict):
 
 
 class Executor(object):
+    _thread_local = threading.local()
     global_env_variables = None
 
+    @classmethod
+    def current(cls):
+        return cls._thread_local.current_executor
+
     def __init__(self, manager, session_config, parent_executor=None):
         self.manager = manager
         self.session_config = session_config
@@ -72,8 +77,9 @@ class Executor(object):
         self.runners = deque()
         self.tasker = None
         if self.global_env_variables is None:
-            self.__class__.global_env_variables = EnvVariables.build_global()
-        self.env_variables = parent_executor.env_variables if parent_executor else EnvVariables(self.global_env_variables)
+            self.__class__.global_env_variables = EnvVariables.build_global(session_config)
+        self.env_variables = EnvVariables(parent_executor.env_variables if parent_executor
+                                          else self.global_env_variables)
 
     def compile_variable(self, sql):
         variables = ENV_VARIABLE_RE.findall(sql)
@@ -88,15 +94,29 @@ class Executor(object):
                     if groups[1]:
                         for arg in groups[1].split(","):
                             calculater_args.append(parse_value(arg))
-                    calculater = find_calculater(groups[0].split("__")[0])(groups[0].replace("__", "::"))
+                    calculater = find_calculater(groups[0].split("__")[0]).instance(groups[0].replace("__", "::"))
                     variable_value = calculater.calculate(*tuple(calculater_args))
                     variable_value = StringFilter().filter(variable_value)
                 except Exception as e:
                     variable_value = default_value[1:]
-            if isinstance(variable_value, str):
-                variable_value = "true" if variable_value is True else ("false" if variable_value is False else
-                                                                        ("null" if variable_value is None else str(variable_value)))
-            sql = sql.replace(variable, variable_value)
+
+            def format_variable_value(value):
+                if value is True:
+                    return "true"
+                if value is False:
+                    return "false"
+                if value is None:
+                    return "null"
+                if isinstance(value, datetime.date):
+                    if isinstance(value, datetime.datetime):
+                        return value.strftime(self.session_config.get().get("datetime_format", "%Y-%m-%d %H:%M:%S"))
+                    return value.strftime(self.session_config.get().get("date_format", "%Y-%m-%d"))
+                if isinstance(value, datetime.time):
+                    return value.strftime(self.session_config.get().get("time_format", "%H:%M:%S"))
+                if isinstance(value, (list, tuple, set)):
+                    return "(" + ",".join([format_variable_value(v) for v in value]) + ")"
+                return str(value)
+            sql = sql.replace(variable, format_variable_value(variable_value))
         return sql
 
     def run(self, name, sqls):
@@ -114,15 +134,18 @@ class Executor(object):
             self.compile(name + "(" + str(lineno) + ")", sql)
 
     def compile(self, name, sql):
-        config = copy.deepcopy(self.session_config.get())
+        config = self.session_config.get()
         config["name"] = name
-        compiler = Compiler(config, self.env_variables)
-        arguments = {"@verbose": self.env_variables.get("@verbose", False), "@timeout": self.env_variables.get("@timeout", 0),
-                     "@limit": self.env_variables.get("@limit", 0), "@batch": self.env_variables.get("@batch", 0),
-                     "@streaming": self.env_variables.get("@streaming", False), "@recovery": self.env_variables.get("@recovery", False),
-                     "@join_batch": self.env_variables.get("@join_batch", 10000), "@insert_batch": self.env_variables.get("@insert_batch", 0),
-                     "@primary_order": False}
-        tasker = compiler.compile(sql, arguments)
+        try:
+            compiler = Compiler(config, self.env_variables)
+            arguments = {"@verbose": self.env_variables.get("@verbose", False), "@timeout": self.env_variables.get("@timeout", 0),
+                         "@limit": self.env_variables.get("@limit", 0), "@batch": self.env_variables.get("@batch", 0),
+                         "@streaming": self.env_variables.get("@streaming", False), "@recovery": self.env_variables.get("@recovery", False),
+                         "@join_batch": self.env_variables.get("@join_batch", 10000), "@insert_batch": self.env_variables.get("@insert_batch", 0),
+                         "@primary_order": False}
+            tasker = compiler.compile(sql, arguments)
+        finally:
+            config["name"] = ""
         self.runners.extend(tasker.start(name, self, self.session_config, self.manager, arguments))
 
     def execute(self):
@@ -145,15 +168,27 @@ class Executor(object):
         self.runners.append(tasker)
 
     def clear_temporary_memory_collection(self, names):
-        for config_key, factory in self.manager.database_manager.factorys.items():
-            if not isinstance(factory, MemoryDBFactory):
-                continue
-            for driver in factory.drivers:
-                if not isinstance(driver.instance, MemoryDBCollection):
+        try:
+            database_config = dict(**[database for database in self.session_config.get()["databases"]
+                                      if database["name"] == "--"][0])
+        except (TypeError, KeyError, IndexError):
+            return
+        database_cls = find_database(database_config.pop("driver"))
+        if not database_cls:
+            return
+        database = database_cls(self.manager.database_manager, database_config).build()
+        try:
+            for name in names:
+                if self.runners and database.is_streaming(name):
                     continue
-                for key in list(driver.instance.keys()):
-                    if key not in names or (self.runners and
-                                            self.manager.database_manager.get_state(
-                                                config_key + "::" + key, "is_streaming")):
-                        continue
-                    driver.instance.remove(key)
+                delete = database.delete(name, ["id"])
+                delete.commit()
+        finally:
+            database.close()
+
+    def __enter__(self):
+        self._thread_local.current_executor = self
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._thread_local.current_executor = self.parent_executor
