@@ -4,7 +4,6 @@
 
 import os
 import sys
-from collections import defaultdict
 from pygments.lexers import SqlLexer
 from prompt_toolkit import PromptSession
 from prompt_toolkit.history import FileHistory
@@ -16,135 +15,132 @@ from prompt_toolkit.cursor_shapes import CursorShape
 from .version import version
 from .parser import SqlParser, SqlSegment
 
-sql_completer = WordCompleter(
-    [
-        "abort",
-        "action",
-        "add",
-        "after",
-        "all",
-        "alter",
-        "analyze",
-        "and",
-        "as",
-        "asc",
-        "attach",
-        "autoincrement",
-        "before",
-        "begin",
-        "between",
-        "by",
-        "cascade",
-        "case",
-        "cast",
-        "check",
-        "collate",
-        "column",
-        "commit",
-        "conflict",
-        "constraint",
-        "create",
-        "cross",
-        "current_date",
-        "current_time",
-        "current_timestamp",
-        "database",
-        "default",
-        "deferrable",
-        "deferred",
-        "delete",
-        "desc",
-        "detach",
-        "distinct",
-        "drop",
-        "each",
-        "else",
-        "end",
-        "escape",
-        "except",
-        "exclusive",
-        "exists",
-        "explain",
-        "fail",
-        "for",
-        "foreign",
-        "from",
-        "full",
-        "glob",
-        "group",
-        "having",
-        "if",
-        "ignore",
-        "immediate",
-        "in",
-        "index",
-        "indexed",
-        "initially",
-        "inner",
-        "insert",
-        "instead",
-        "intersect",
-        "into",
-        "is",
-        "isnull",
-        "join",
-        "key",
-        "left",
-        "like",
-        "limit",
-        "match",
-        "natural",
-        "no",
-        "not",
-        "notnull",
-        "null",
-        "of",
-        "offset",
-        "on",
-        "or",
-        "order",
-        "outer",
-        "plan",
-        "pragma",
-        "primary",
-        "query",
-        "raise",
-        "recursive",
-        "references",
-        "regexp",
-        "reindex",
-        "release",
-        "rename",
-        "replace",
-        "restrict",
-        "right",
-        "rollback",
-        "row",
-        "savepoint",
-        "select",
-        "set",
-        "table",
-        "temp",
-        "temporary",
-        "then",
-        "to",
-        "transaction",
-        "trigger",
-        "union",
-        "unique",
-        "update",
-        "using",
-        "vacuum",
-        "values",
-        "view",
-        "virtual",
-        "when",
-        "where",
-        "with",
-        "without",
-    ],
-    ignore_case=True,
-)
+SQL_COMPLETER_WORDS = [
+    "abort",
+    "action",
+    "add",
+    "after",
+    "all",
+    "alter",
+    "analyze",
+    "and",
+    "as",
+    "asc",
+    "attach",
+    "autoincrement",
+    "before",
+    "begin",
+    "between",
+    "by",
+    "cascade",
+    "case",
+    "cast",
+    "check",
+    "collate",
+    "column",
+    "commit",
+    "conflict",
+    "constraint",
+    "create",
+    "cross",
+    "current_date",
+    "current_time",
+    "current_timestamp",
+    "database",
+    "default",
+    "deferrable",
+    "deferred",
+    "delete",
+    "desc",
+    "detach",
+    "distinct",
+    "drop",
+    "each",
+    "else",
+    "end",
+    "escape",
+    "except",
+    "exclusive",
+    "exists",
+    "explain",
+    "fail",
+    "for",
+    "foreign",
+    "from",
+    "full",
+    "glob",
+    "group",
+    "having",
+    "if",
+    "ignore",
+    "immediate",
+    "in",
+    "index",
+    "indexed",
+    "initially",
+    "inner",
+    "insert",
+    "instead",
+    "intersect",
+    "into",
+    "is",
+    "isnull",
+    "join",
+    "key",
+    "left",
+    "like",
+    "limit",
+    "match",
+    "natural",
+    "no",
+    "not",
+    "notnull",
+    "null",
+    "of",
+    "offset",
+    "on",
+    "or",
+    "order",
+    "outer",
+    "plan",
+    "pragma",
+    "primary",
+    "query",
+    "raise",
+    "recursive",
+    "references",
+    "regexp",
+    "reindex",
+    "release",
+    "rename",
+    "replace",
+    "restrict",
+    "right",
+    "rollback",
+    "row",
+    "savepoint",
+    "select",
+    "set",
+    "table",
+    "temp",
+    "temporary",
+    "then",
+    "to",
+    "transaction",
+    "trigger",
+    "union",
+    "unique",
+    "update",
+    "using",
+    "vacuum",
+    "values",
+    "view",
+    "virtual",
+    "when",
+    "where",
+    "with",
+    "without",
+]
 
 style = Style.from_dict(
     {
@@ -201,9 +197,12 @@ class CliPrompt(object):
         self.executor = executor
 
     def run(self):
-        home_config_path = os.path.join(os.path.expanduser('~'), ".syncany")
+        home_config_path = self.session_config.get_home()
         if not os.path.exists(home_config_path):
             os.mkdir(home_config_path)
+        database_completer_words = [database["name"] for database in (self.session_config.get().get("databases") or [])
+                                    if database["name"] not in ("-", "--")]
+        sql_completer = WordCompleter(SQL_COMPLETER_WORDS + database_completer_words, ignore_case=True)
         history = CliFileHistory(os.path.join(home_config_path, "history"))
         session = PromptSession(
             lexer=PygmentsLexer(SqlLexer), completer=sql_completer, style=style, history=history
@@ -237,7 +236,7 @@ class CliPrompt(object):
             except EOFError:
                 return 130
             except Exception as e:
-                print(str(e))
+                print(e.__class__.__name__ + ": " + str(e))
         return 0
 
     def check_complete(self, content):
