@@ -6,24 +6,25 @@ import copy
 import datetime
 from syncany.main import beautify_print
 
+
 class ExplainTasker(object):
-    def __init__(self, tasker):
+    def __init__(self, tasker, sql=None):
         self.config = None
         self.tasker = tasker
+        self.sql = sql
 
     def start(self, name, executor, session_config, manager, arguments):
         self.config = copy.deepcopy(self.tasker.config)
+        if self.sql:
+            beautify_print("%s tasker %s execute sql:\n%s" % (datetime.datetime.now(), self.config["name"], self.sql))
+            print()
         beautify_print("%s tasker %s compiled config:" % (datetime.datetime.now(), self.config["name"]))
         beautify_print(self.config)
         print()
 
         for key in list(arguments.keys()):
-            if key.endswith("@limit"):
-                arguments[key] = 1
-            elif key.endswith("@verbose"):
+            if key.endswith("@verbose"):
                 arguments[key] = True
-        self.tasker.config["output"] = "&.-.&1::" + self.tasker.config["output"].split("::")[-1].split(" ")[0]
-        self.tasker.config["name"] = self.tasker.config["name"] + "#explain"
         self.tasker.start(name, executor, session_config, manager, arguments)
         return [self]
 
