@@ -4,6 +4,7 @@
 
 import os
 import copy
+import time
 import traceback
 import uuid
 from syncany.logger import get_logger
@@ -31,12 +32,12 @@ class ReduceHooker(Hooker):
         self.batch_count = 0
 
     def outputed(self, tasker, datas):
-        self.count += len(datas)
-        self.batch_count += 1
-        if self.batch > 0 and self.count >= self.batch:
+        if self.batch > 0 and self.count > 0 and self.count + len(datas) >= self.batch:
             self.count, self.batch_count = 0, 1
             self.tasker.run_reduce(self.executor, self.session_config, self.manager, self.arguments, False)
         else:
+            self.count += len(datas)
+            self.batch_count += 1
             limit = tasker.arguments["@limit"] if "@limit" in tasker.arguments and tasker.arguments["@limit"] > 0 else 0
             if 0 < limit <= tasker.status["store_count"]:
                 self.count, self.batch_count = 0, 1
@@ -49,8 +50,10 @@ class ReduceHooker(Hooker):
 
 
 class QueryTasker(object):
-    def __init__(self, config):
+    def __init__(self, config, sql_expression=None):
+        self.name = config.get("name", "")
         self.config = config
+        self.sql_expression = sql_expression
         self.reduce_config = None
         self.tasker = None
         self.dependency_taskers = []
@@ -58,17 +61,17 @@ class QueryTasker(object):
         self.tasker_generator = None
         self.updaters = []
         self.temporary_memory_collections = set([])
+        self.run_start_time = 0
 
     def start(self, name, executor, session_config, manager, arguments):
-        dependency_taskers, aggregate = [], self.config.pop("aggregate", None)
+        dependency_taskers, where_schema, aggregate = [], self.config.get("where_schema", None), self.config.pop("aggregate", None)
         if aggregate and aggregate.get("distinct_keys"):
-            self.config, distinct_config = self.compile_distinct_config(aggregate), self.config
+            self.config, distinct_config = self.compile_distinct_config(where_schema, aggregate), self.config
             distinct_config["name"] = distinct_config["name"] + "#select@distinct"
             distinct_tasker = QueryTasker(distinct_config)
             distinct_tasker.start(name, executor, session_config, manager, copy.deepcopy(arguments))
             dependency_taskers.append(distinct_tasker)
-            arguments["@limit"], arguments["@batch"] = 0, 0
-            arguments["@primary_order"] = False
+            arguments["@limit"] = 0
 
         for dependency_config in self.config.get("dependencys", []):
             kn, knl = (dependency_config["name"] + "@"), len(dependency_config["name"] + "@")
@@ -82,6 +85,7 @@ class QueryTasker(object):
             dependency_tasker.start(name, executor, session_config, manager, dependency_arguments)
             dependency_taskers.append(dependency_tasker)
 
+        where_schema = self.config.pop("where_schema", None)
         limit, batch = int(arguments.get("@limit", 0)), int(arguments.get("@batch", 0))
         require_reduce, reduce_intercept, sorted_limit = False, False, len(self.config.get("pipelines", [])) == 2
         if self.config.get("intercepts"):
@@ -96,18 +100,21 @@ class QueryTasker(object):
             elif [aggregate_column["final_value"] for aggregate_column in aggregate["schema"].values()
                   if aggregate_column["final_value"]]:
                 require_reduce = True
+        if where_schema:
+            require_reduce = True
         if require_reduce and isinstance(self.config["schema"], dict) and not arguments.get("@streaming"):
-            if limit > 0 and batch <= 0:
-                batch = min(max(*(int(arguments.get(key, 0)) for key in ("@limit", "@batch", "@join_batch"))), 1000)
-                arguments["@batch"] = batch
             if not aggregate:
                 aggregate = copy.deepcopy(DEAULT_AGGREGATE)
-            self.compile_reduce_config(aggregate)
+            self.compile_reduce_config(where_schema, aggregate)
             if reduce_intercept:
-                self.reduce_config["intercepts"] = self.config.pop("intercepts", [])
+                intercepts = self.config.pop("intercepts", [])
+                self.config["intercepts"], self.reduce_config["intercepts"] = intercepts[:1], intercepts[1:]
             self.reduce_config["pipelines"] = self.config.pop("pipelines", [])
         elif 0 < limit < batch:
             arguments["@batch"] = limit
+        if "intercepts" in self.config:
+            self.config["intercepts"] = [intercept for intercept in self.config["intercepts"] if
+                                         intercept != ["#const", True]]
         tasker = CoreTasker(self.config, manager)
         if "#" not in tasker.config["name"]:
             tasker.config["name"] = tasker.config["name"] + "#select"
@@ -123,6 +130,7 @@ class QueryTasker(object):
             self.execute_updater(executor, session_config, manager)
             if not self.tasker_generator:
                 self.tasker_generator = self.run_tasker(executor, session_config, manager, self.tasker, self.dependency_taskers)
+                self.run_start_time = time.time()
             else:
                 _thread_local.current_tasker = self.tasker
             while True:
@@ -134,10 +142,16 @@ class QueryTasker(object):
                 except StopIteration as e:
                     exit_code = e.value
                     if exit_code is not None and exit_code != 0:
+                        if self.sql_expression:
+                            get_logger().info("execute SQL %s finish with code %s %.2fms", self.name, exit_code,
+                                              (time.time() - self.run_start_time) * 1000)
                         return exit_code
                     break
             if self.reduce_config:
-                return self.run_reduce(executor, session_config, manager, self.arguments, True)
+                exit_code = self.run_reduce(executor, session_config, manager, self.arguments, True)
+            if self.sql_expression:
+                get_logger().info("execute SQL %s finish with code %s %.2fms", self.name, exit_code,
+                                  (time.time() - self.run_start_time) * 1000)
             return exit_code
         finally:
             names = self.get_temporary_memory_collections()
@@ -175,9 +189,11 @@ class QueryTasker(object):
         self.tasker.terminate()
         self.tasker = None
 
-    def compile_distinct_config(self, aggregate):
+    def compile_distinct_config(self, where_schema, aggregate):
         subquery_name = "__subquery_" + str(uuid.uuid1().int) + "_distinct"
         config = copy.deepcopy(self.config)
+        config.pop("loader", None)
+        config.pop("loader_arguments", None)
         config.update({
             "input": "&.--." + subquery_name + "::" + self.config["output"].split("::")[-1].split(" ")[0],
             "output": self.config["output"],
@@ -214,11 +230,14 @@ class QueryTasker(object):
                 distinct_aggregate["schema"][key] = copy.deepcopy(aggregate["schema"][key])
                 distinct_aggregate["schema"][key]["final_value"] = None
                 config["schema"][key] = ["#aggregate", "$._aggregate_distinct_key_", aggregate["schema"][key]["reduce"]]
+            elif where_schema and key in where_schema:
+                continue
             else:
                 config["schema"][key] = "$." + key
 
         if aggregate["key"]:
             self.config["schema"]["_aggregate_distinct_key_"] = aggregate["key"]
+            aggregate["key"] = "$._aggregate_distinct_key_"
         self.config["schema"]["_aggregate_distinct_aggregate_key_"] = ["#aggregate", group_column, ["#const", 0]]
         distinct_aggregate["key"] = group_column
         distinct_aggregate["schema"]["_aggregate_distinct_aggregate_key_"] = {
@@ -231,16 +250,25 @@ class QueryTasker(object):
         }
         self.config["aggregate"] = distinct_aggregate
         if [having_column for having_column in aggregate["having_columns"] if having_column in aggregate["schema"]]:
-            config["intercepts"] = self.config.pop("intercepts", [])
+            intercepts = self.config.pop("intercepts", [])
+            if intercepts and len(intercepts) >= 1:
+                self.config["intercepts"] = [intercepts[0], ["#const", True]]
+            if intercepts and len(intercepts) >= 2:
+                config["intercepts"] = [["#const", True], intercepts[1]]
         else:
             distinct_aggregate["having_columns"] = aggregate["having_columns"]
+        config.pop("where_schema", None)
         config["pipelines"] = self.config.pop("pipelines", [])
         self.config["output"] = "&.--." + subquery_name + "::" + self.config["output"].split("::")[-1].split(" ")[0] + " use I"
+        self.config.pop("outputer", None)
+        self.config.pop("outputer_arguments", None)
         return config
 
-    def compile_reduce_config(self, aggregate):
+    def compile_reduce_config(self, where_schema, aggregate):
         subquery_name = "__subquery_" + str(uuid.uuid1().int) + "_reduce"
         config = copy.deepcopy(self.config)
+        config.pop("loader", None)
+        config.pop("loader_arguments", None)
         config.update({
             "input": "&.--." + subquery_name + "::" + self.config["output"].split("::")[-1].split(" ")[0],
             "output": self.config["output"],
@@ -267,27 +295,36 @@ class QueryTasker(object):
             config["schema"]["_aggregate_key_"] = "$._aggregate_key_"
             self.config["schema"]["_aggregate_key_"] = aggregate["key"]
         config["aggregate"] = aggregate
+        config["where_schema"] = where_schema
         self.config["output"] = "&.--." + subquery_name + "::" + self.config["output"].split("::")[-1].split(" ")[0] + " use I"
+        self.config.pop("outputer", None)
+        self.config.pop("outputer_arguments", None)
         self.reduce_config = config
 
     def run_reduce(self, executor, session_config, manager, arguments, final_reduce=False):
         config, arguments = copy.deepcopy(self.reduce_config), copy.deepcopy(arguments)
+        where_schema, aggregate = config.pop("where_schema", None), config.pop("aggregate", None)
         if final_reduce:
             config["schema"].pop("_aggregate_key_", None)
+            if where_schema:
+                for key in where_schema:
+                    config.pop(key, None)
             for key, column in config["schema"].items():
-                if key in config["aggregate"]["schema"] and config["aggregate"]["schema"][key]["final_value"]:
-                    config["schema"][key] = config["aggregate"]["schema"][key]["final_value"]
+                if key in aggregate["schema"] and aggregate["schema"][key]["final_value"]:
+                    config["schema"][key] = aggregate["schema"][key]["final_value"]
                 else:
                     config["schema"][key] = "$." + key
-            config["name"] = config["name"] + "#select@final_reduce"
+            config["name"] = config["name"] + "#select@final"
         else:
             config["output"] = config["input"] + " use DI"
             config["name"] = config["name"] + "#select@reduce"
             config["intercepts"] = []
             config["pipelines"] = []
+        if "intercepts" in config:
+            config["intercepts"] = [intercept for intercept in config["intercepts"] if
+                                    intercept != ["#const", True]]
         tasker = CoreTasker(config, manager)
         arguments["@primary_order"] = False
-        arguments["@batch"] = 0
         arguments["@limit"] = 0
         self.compile_tasker(arguments, tasker)
         tasker_generator = self.run_tasker(executor, session_config, manager, tasker, [])
@@ -308,6 +345,11 @@ class QueryTasker(object):
         compile_arguments = {}
         compile_arguments.update({key.lower(): value for key, value in os.environ.items()})
         compile_arguments.update(arguments)
+        if self.is_local_memory_database(tasker.config):
+            if isinstance(tasker.config["input"], str) and "&.--." in tasker.config["input"]:
+                compile_arguments["@batch"] = 0
+            if isinstance(tasker.config["output"], str) and "&.--." in tasker.config["output"]:
+                compile_arguments["@insert_batch"] = 0
 
         tasker.compile(compile_arguments)
         if "@verbose" in compile_arguments and compile_arguments["@verbose"]:
@@ -378,3 +420,11 @@ class QueryTasker(object):
         for dependency_tasker in self.dependency_taskers:
             names.extend(dependency_tasker.get_temporary_memory_collections())
         return names
+
+    def is_local_memory_database(self, config, name="--"):
+        try:
+            database_config = dict(**[database for database in config["databases"]
+                                      if database["name"] == name][0])
+            return database_config["driver"] == "memory"
+        except (TypeError, KeyError, IndexError):
+            return False
